# frozen_string_literal: true

class OrdersController < ApplicationController
  model_data_store(Order)
  def create
    @model = Order.submit_order!(model_params.to_h)
    @model.submitted_by = current_user[:name]
    @model.origin = 'RXSITE'
    @model.save
    render json: @model, status: :created, location: @model
  end

  def duplicate?
    result = Order.duplicate_order?(params[:po_number], params[:customer_id])
    render json: { result: result }
  end

  private

  # Only allow a list of trusted parameters through.
  def model_params
    params.require(:order).permit(
      :_fmupc, :_fsku, :_fstatus, :customer_city, :customer_name, :customer_num, :date_completed, :date_ordered, :date_received, :fcol, :fram, :fupc, :job_type, :lens_color, :lens_treat, :order_number, :po_number, :raw_data, :release_number, :ship_to_address1, :ship_to_address2, :ship_to_address3, :ship_to_address4, :ship_to_code, :ship_to_name, :ship_to_num, :tray, :cust_job_id, :trace_id,
      :style_type_id, :style_id, :style_material_id, :customer_id, :shipping_methods_id, :frame_sku, :hold_at_lab, :comments, :submitted_by, addon_ids: [], raw_data: {}
    )
  end

  def fix_params(param)
    param[:id] = 'styles.style' if param[:id] == 'style_name'
    param[:id] = 'style_types.style_type' if param[:id] == 'style_type_name'
    param[:id] = 'style_materials.material_description' if param[:id] == 'style_material_name'

    param
  end

  def before_sort(sort)
    fix_params(sort)
  end

  def before_filter(filter)
    if filter[:id] == 'customer_id'
      filter[:strict_match] = true
      filter[:value] = filter[:value].split('|')
    end

    filter[:strict_match] = true if filter[:id] == 'hold_at_lab'
    filter[:strict_match] = true if filter[:id] == 'is_valid'
    filter[:strict_match] = true if filter[:id] == 'is_shipped'

    if filter[:id] == "job_track_station"
      filter[:strict_match] = true
    end

    fix_params(filter)
  end

  def before_set_targets
    @models = if RxModule::Authorization.current_user_can?('config', 'user_config_admin')
                @models
              else
                config = Config.find_by(user_id: RxModule::Authorization.current_user[:id])
                @models = @models.where(customer_id: config.config_customers.map(&:customer_id))
              end
    @models.eager_load(:style, :style_type, :style_material, :customer)
  end
end
