import * as React from 'react';
import {ModelSelect, useModelContext, useReadonlyContext} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {AddonModel, AddonStore} from '../../../stores/AddonStore';
import {sortedOptions} from "../../../stores/_FieldConstants";

const FILTER_KEYS = ["ar", "tint", "mirror", "backside", "uv", "harden", "dropball", "etch", "polish"];

export const AddonSelect = observer(() => {
    const model = useModelContext();
    const readOnly = useReadonlyContext();

    React.useEffect(() => {
        AddonStore.getData();
    }, []);

    const addons = AddonStore.models as AddonModel[];
    const ids = model?.viewModel?.addon_ids as string[];

    const filters = React.useMemo(() => {
        const tmp: { [key: string]: boolean } = {
            ar: false,
            tint: false,
            mirror: false,
            backside: false,
            uv: false,
            harden: false,
            dropball: false,
            etch: false,
            polish: false
        };


        ids?.forEach(addon_id => {
            const addon = AddonStore.get(addon_id);
            if (addon) {
                FILTER_KEYS.forEach(key => {
                    if (addon[key]) {
                        tmp[key] = true;
                    }
                })
            }
        })

        return tmp;
    }, [ids])


    const options = addons.filter(a => {
        return !FILTER_KEYS.some(key => {
            const shouldBeFiltered = a[key] && filters[key]
            const alreadySelected = ids?.includes(a.getId());
            return shouldBeFiltered && !alreadySelected;
        });
    }).map(a => {
        let label;
        if (a.desc) {
            label = `${a.code} - ${a.desc}`
        } else {
            label = a.code;
        }
        return {
            label,
            value: a.getId()
        }
    })

    return (
        <ModelSelect
            name={"addon_ids"}
            disabled={readOnly}
            options={sortedOptions(options)}
            SelectProps={{
                tabSelectsValue: false
            }}
            multiple
        />
    )
});