import * as React from 'react';
import {observer} from "mobx-react-lite";
import {InputField, ModelInput, useModelContext, useReadonlyContext} from "@rx-suite/ui";
import {LabMaintenanceStore} from "../../../stores/LabMaintenanceStore";
import { Grid } from '@material-ui/core';
import { t } from '@lingui/macro';

export const CustomerNameField = observer(() => {
    const model = useModelContext();
    const readOnly = useReadonlyContext();
    const customerNameInput = React.useRef<HTMLInputElement>();

    const handleBlur = React.useCallback(() => {
        if (!model.viewModel.ship_to_name) {
            model.viewModel.ship_to_name = model.viewModel.customer_name;
        }
    }, [model.viewModel]);

    React.useEffect(()=>{
        customerNameInput?.current?.focus();
    },[]);

    const [ state, setState ] = React.useState({
        firstName: "",
        lastName: ""
    })

    React.useEffect(() => {
        if(model?.customer_name) {
            const nameParts = (model.customer_name as string).split(" ");
            setState({
                lastName: nameParts.splice(1).join(" "),
                firstName: nameParts[0],

            })
        }

    }, [model.customer_name]);

    const handleSplitNameChange = React.useCallback(e => {
        const { name, value } = e.target;

        setState(prevState => {
            const newState = {
                ...prevState,
                [name]: value,
            }
            model.viewModel.customer_name = `${newState.firstName} ${newState.lastName}`;

            return newState;
        });
    }, [model.viewModel]);

    if(LabMaintenanceStore.selectedModel?.split_name_on_order_entry) {
        return (
            <Grid container spacing={2}>
                <Grid item xs>
                    <InputField
                        name={"firstName"}
                        label={t`First Name`}
                        value={state.firstName}
                        fullWidth
                        disabled={readOnly}
                        onChange={handleSplitNameChange}
                        inputRef={customerNameInput}
                    />
                </Grid>
                <Grid item xs>
                    <InputField
                        name={"lastName"}
                        fullWidth
                        value={state.lastName}
                        label={t`Last Name`}
                        disabled={readOnly}
                        onBlur={handleBlur}
                        onChange={handleSplitNameChange}
                    />

                </Grid>
            </Grid>
        )
    } else {
        return (
            <ModelInput
                name={"customer_name"}
                onBlur={handleBlur}
                fullWidth
                inputRef={customerNameInput}
            />
        )
    }


})