import * as React from 'react';
import {Grid} from '@material-ui/core';
import {observer} from 'mobx-react-lite';
import {GeneratedField} from './GeneratedField';
import {useOrderEntryStore} from "../../../hooks/useOrderEntryStore";
import {LevelHeader} from "./LevelHeader";
import {LevelWrapper} from "./LevelWrapper";
import {ExcludedFieldKeysForEdgeOnlyJobType, Field} from "../../../stores/_FieldConstants";
import {CurrentUserConfig} from "../../../stores/ConfigStore";

export const PerJobFields = observer(() => {
    const store = useOrderEntryStore();
    const groups: { [index: number]: Field[] } = {};
    const isEdgeOnlyJob = store.order?.isEdgeOnlyJob || false;

    if (isEdgeOnlyJob) {
        store.perJob.filter(f => !ExcludedFieldKeysForEdgeOnlyJobType.includes(f.name)).forEach(f => {
            groups[f.level] = groups[f.level] || [];
            groups[f.level].push(f);
        });
    } else {
        store.perJob!.forEach(f => {
            groups[f.level] = groups[f.level] || [];
            groups[f.level].push(f);
        });
    }

    if(!CurrentUserConfig.get().allow_personalization) {
        delete groups[9];
    }

    return (
        <>
            {
                Object.entries(groups).map(([level, arr]) => (
                    <LevelWrapper key={level} level={level}>
                        <Grid item xs={12}>
                            <Grid container spacing={4}>
                                <Grid item xs={12} lg={12}>
                                    <LevelHeader level={level}/>
                                </Grid>
                                <Grid item xs={12}>
                                    <Grid container spacing={4}>
                                        {
                                            arr.map(f => (
                                                <Grid key={"raw_data." + f.name} item xs={12} lg={2}>
                                                    <GeneratedField
                                                        label={f.desc}
                                                        name={"raw_data." + f.name}
                                                        isSelect={!f.increment}
                                                        options={f.options}
                                                        field={f}
                                                    />
                                                </Grid>
                                            ))
                                        }
                                    </Grid>
                                </Grid>
                            </Grid>
                        </Grid>
                    </LevelWrapper>
                ))
            }
        </>
    )
});
