import * as React from "react";
import {observer} from "mobx-react-lite";
import {ModelInput, useModelContext} from "@rx-suite/ui";
import {useOrderEntryStore} from "../../../hooks/useOrderEntryStore";
import {t} from "@lingui/macro";

export const PurchaseOrderInputField = observer(() => {
    const model = useModelContext();
    const store = useOrderEntryStore();
    const isVA = store.order?.isVA;
    const resolvedLabel = isVA ? t`Consult Number` : t`PO Number`
    const resolvedHint ={title: resolvedLabel, text: t`${isVA ? "Number used for external tracking" : "The Purchase Order number associated to this order"}`}

    const handleBlur = React.useCallback(async () => {
        if (model.viewModel.dig("po_number").trim()){
            const isDuplicate = await store.checkDuplicateOrder();
            if (isDuplicate){
                model.errors["po_number"] = t`An open order with the same ${resolvedLabel} already exists for the customer. Please update it if this is not a duplicate order`
            }
        }
    }, [store, model.errors, model.viewModel?.po_number]);

    return (
        <ModelInput name={"po_number"}
                    onBlur={handleBlur}
                    label={resolvedLabel}
                    hint={resolvedHint}
        />
    );
})