import {observer} from 'mobx-react-lite';
import * as React from 'react';
import {t} from "@lingui/macro";
import {useOrderEntryStore} from "../../../hooks/useOrderEntryStore";
import LoadingOverlay from "react-loading-overlay";
import {ReadonlyProvider} from "@rx-suite/ui";
import {FormHelperText} from "@material-ui/core";
import {useSnackbar} from "notistack";

export const FrameWrapper = observer(({children}) => {
    const store = useOrderEntryStore();
    const {enqueueSnackbar} = useSnackbar();
    const fetchingFrameCandidatesError = store.fetchingFrameCandidatesError;
    const fetchingFrameTraceError = store.fetchingFrameTraceError;

    React.useEffect(() => {
        if (store.fetchingFrameCandidatesError) {
            enqueueSnackbar(store.fetchingFrameCandidatesError, {variant: 'error'});
        }
    }, [enqueueSnackbar, store.fetchingFrameCandidatesError]);
    {/*// @ts-ignore the types declaration doesn't include children*/
    }
    return <LoadingOverlay
        active={store.fetchingFrameCandidates}
        text={t`Looking for frames...`}
        spinner
    >
        {fetchingFrameCandidatesError && <FormHelperText error>{fetchingFrameCandidatesError}</FormHelperText>}
        {fetchingFrameTraceError && <FormHelperText error>{fetchingFrameTraceError}</FormHelperText>}
        <ReadonlyProvider value={store.lockFrameFields}>
            {children}
        </ReadonlyProvider>
    </LoadingOverlay>
});