import * as React from 'react';
import { observer } from 'mobx-react-lite';
import { useOrderEntryStore } from '../../../hooks/useOrderEntryStore';
import Two from 'two.js';
import {Box, Grid, Slider, Typography} from "@material-ui/core";
import {runInAction} from "mobx";
import {useRef} from "react";
import TextField from "@material-ui/core/TextField";

export const LensPreview = observer(() => {
    const store = useOrderEntryStore();
    const trace = store.frameTrace;

    const changed = useRef(false);

    const handleFieldChange = React.useCallback(e => {
        store.viewModel.frameTrace = { ...store.viewModel.frameTrace, [e.target.name]: e.target.value};
    }, [store.viewModel])

    const handleAChange = React.useCallback((e, a) => {
        runInAction(() => {
            store.viewModel.frameTrace = { ...store.viewModel.frameTrace, a };
        })
    }, [store.viewModel]);

    const handleBChange = React.useCallback((e, b) => {
        runInAction(() => {
            store.viewModel.frameTrace = { ...store.viewModel.frameTrace, b };
        })
    }, [store.viewModel]);

    const handleCChange = React.useCallback((e, circ) => {
        runInAction(() => {
            store.viewModel.frameTrace = { ...store.viewModel.frameTrace, circ };
        })
    }, [store.viewModel]);

    const handleEdChange = React.useCallback((e, ed) => {
        runInAction(() => {
            store.viewModel.frameTrace = { ...store.viewModel.frameTrace, ed };
        })
    }, [store.viewModel]);

    const handleDblChange = React.useCallback((e, dbl) => {
        runInAction(() => {
            store.viewModel.frameTrace = { ...store.viewModel.frameTrace, dbl };
        })
    }, [store.viewModel]);

    const body = React.useRef<HTMLDivElement>();

    React.useEffect(() => {
        const scale = 3.582677165; // 91ppi

        let a = store?.frameTrace?.viewModel?.a || 0;
        let b = store?.frameTrace?.viewModel?.b || 0;
        let dbl = store?.frameTrace?.viewModel?.dbl || 0;

        let originalA = a;
        let originalB = b;

        const two = new Two({ autostart: true }).appendTo(body.current);
        two.width = 858;
        two.height = 400;

        let data = store.frameTrace.radii;

        const degToRad = (deg: number) => {
            return (deg * Math.PI) / 180;
        }

        const xScale = () => {
            return (a / originalA) || 1;
        }

        const yScale = () => {
            return (b / originalB) || 1;
        }

        const calcX = (distance, angle) => {
            return (distance * Math.cos(angle) * xScale());
        }

        const calcY = (distance, angle) => {
            return (distance * Math.sin(angle) * yScale());
        }

        const calcAngle = (angles, radii, index) => {
            if(angles.length) {
                return degToRad(angles[index]);
            } else {
                return degToRad((radii.length - index) * (360 / radii.length));
            }
        }

        const createPlotData = () => {
            const radii = data && data.radii || data || [];
            const angles = data && data.angles || [];
            const arr = [];

            for(let i = 0; i < radii.length; i++) {
                const distance = parseFloat(radii[i]);
                const angle = calcAngle(angles, radii, i);
                const anchor = new Two.Anchor(calcX(distance, angle), calcY(distance, angle));
                arr.push(anchor);
            }

            return arr;
        }

        const calculateMinMax = (data) => {
            const minMax = [];
            for(let i = 0; i < data.length; i++) {
                if(minMax.length) {
                    minMax[0] = Math.max(minMax[0], data[i].x);
                    minMax[1] = Math.max(minMax[1], data[i].y);
                    minMax[2] = Math.min(minMax[2], data[i].x);
                    minMax[3] = Math.min(minMax[3], data[i].y);
                } else {
                    minMax[0] = data[i].x;
                    minMax[1] = data[i].y;
                    minMax[2] = data[i].x;
                    minMax[3] = data[i].y;
                }
            }
            return minMax;
        }

        const distanceBetween = (a, b) => {
            return Math.sqrt(Math.pow(b.x - a.x, 2) + Math.pow(b.y - a.y, 2));
        };

        const fixFloat = (f) => {
            return (Math.round(f * 10) / 10).toFixed(2);
        };

        const performCalcs = (vertices) => {
            const minMax = calculateMinMax(vertices);

            const center = {
                x: (minMax[0] + minMax[2]) / 2,
                y: (minMax[1] + minMax[3]) / 2
            };

            let maxDistance = 0;
            let circ = 0;
            let lastPoint = vertices[0];
            for(let i = 0; i < vertices.length; i++) {
                const distance = distanceBetween(vertices[i], center);
                maxDistance = Math.max(distance, maxDistance);
                circ += distanceBetween(vertices[i], lastPoint);
                lastPoint = vertices[i];
            }

            return {
                center: center,
                boundingBox: [
                    new Two.Anchor(minMax[0], minMax[1]),
                    new Two.Anchor(minMax[2], minMax[1]),
                    new Two.Anchor(minMax[2], minMax[3]),
                    new Two.Anchor(minMax[0], minMax[3])
                ],
                ed: fixFloat(maxDistance * 2),
                circ: fixFloat(circ + distanceBetween(vertices[0], vertices[vertices.length - 1]))
            }

        }

        const createDBLBox = (a, b, dbl) => {
            const right = parseFloat(dbl) / 2;
            const left = -right;
            const top = b / 4;
            const bot = -top;
            return [
                new Two.Anchor(right, top),
                new Two.Anchor(right, bot),
                new Two.Anchor(left, bot),
                new Two.Anchor(left, top)
            ];
        };


        const updateVertices = () => {
            const vertices = createPlotData();
            const calcs = performCalcs(vertices);

            const center = calcs.center;//calculateGeometricCenter(vertices);
            const boundingBoxVertices = calcs.boundingBox;//createBoundingBox(vertices);
            const xOffset = (((parseFloat(a)) / 2) * scale) + ((parseFloat(dbl) / 2) * scale) - (center.x * scale);
            const yScale = (data && data.angles || []).length ? -1 : 1;

            // $('#ed-frame-slider')[0].noUiSlider.set(calcs.ed);
            // $('#c-frame-slider')[0].noUiSlider.set(calcs.circ);

            store.viewModel.frameTrace.ed = calcs.ed;
            store.viewModel.frameTrace.circ = calcs.circ;

            const left = two.makePath(vertices);
            left.translation = new Two.Vector(((two.width) / 2) + xOffset, ((two.height) / 2) + center.y);
            left.scale = new Two.Vector(-scale, scale * yScale);
            left.linewidth = 0.2;


            const leftBox = two.makePath(boundingBoxVertices);
            leftBox.translation = new Two.Vector((two.width / 2) - (xOffset), (two.height / 2) + center.y);
            leftBox.scale = new Two.Vector(-scale, scale * yScale);
            leftBox.noFill();
            leftBox.stroke = "black";
            leftBox.linewidth = 0.2;
            leftBox.dashes = [2,2];

            const right = two.makePath(vertices);
            right.translation = new Two.Vector(((two.width) / 2) - xOffset, ((two.height) / 2) + center.y);
            right.scale = new Two.Vector(scale, scale * yScale);
            right.linewidth = 0.2;

            const rightBox = two.makePath(boundingBoxVertices);
            rightBox.translation = new Two.Vector((two.width / 2) + (xOffset), (two.height / 2) + center.y);
            rightBox.scale = new Two.Vector(scale, scale * yScale);
            rightBox.linewidth = 0.2;
            rightBox.dashes = [2,2];
            rightBox.noFill();

            const dblBox = two.makePath(createDBLBox(a, b, dbl));
            dblBox.scale = scale;
            dblBox.linewidth = 0.2;
            dblBox.translation = new Two.Vector(two.width / 2, two.height / 2);
            dblBox.noFill();
            dblBox.dashes = [2,2];

           // left.fill = two.makeRadialGradient(10, -25, a * 2);
           // right.fill = two.makeRadialGradient(-10, -25, a * 2);
        }

        two.bind("update", () => {
            if(changed.current && trace) {
                two.clear();

                const background = two.makeRectangle(two.width / 2, two.height / 2, two.width, two.height);
                background.fill = "#5FA4FF";

                data = store?.frameTrace?.radii;
                a = store?.viewModel?.frameTrace?.a;
                b = store?.viewModel?.frameTrace?.b;
                dbl = store?.viewModel?.frameTrace?.dbl;

                originalA = store?.frameTrace?.a;
                originalB = store?.frameTrace?.b;

                changed.current = false;
                updateVertices();
            }
        });

        return () => {
            two.unbind("update");
            two.pause();
            body.current.removeChild(two.renderer.domElement);
        }
    }, []);

    React.useEffect(() => {
        changed.current = true;
    }, [store.viewModel.frameTrace?.radii, store.viewModel.frameTrace?.a, store.viewModel.frameTrace?.b, store.viewModel.frameTrace?.circ, store.viewModel.frameTrace?.dbl, store.viewModel.frameTrace?.ed]);

    if(!trace) {
        return <Box display="flex" justifyContent="center" ref={body}></Box>;
    }

    return(
        <>
            <Box display="flex" justifyContent="center" ref={body}></Box>
            <Grid container spacing={2}>
                <Grid item xs>
                    <Typography gutterBottom>
                        A
                    </Typography>
                    <Grid container>
                        <Grid item xs>
                            <Slider value={store.viewModel.frameTrace.a} min={0} onChange={handleAChange} />
                            <TextField value={store.viewModel.frameTrace.a} name={"a"} onChange={handleFieldChange} />
                        </Grid>
                    </Grid>
                </Grid>
                <Grid item xs>
                    <Typography gutterBottom>
                        B
                    </Typography>
                    <Grid container>
                        <Grid item xs>
                            <Slider value={store.viewModel.frameTrace.b} min={0} onChange={handleBChange} />
                            <TextField value={store.viewModel.frameTrace.b} name={"b"} onChange={handleFieldChange} />
                        </Grid>
                    </Grid>
                </Grid>
                <Grid item xs>
                    <Typography gutterBottom>
                        ED
                    </Typography>
                    <Grid container>
                        <Grid item xs>
                            <Slider value={store.viewModel.frameTrace.ed} min={0} disabled onChange={handleEdChange} />
                            <TextField value={store.viewModel.frameTrace.ed} disabled name={"ed"} onChange={handleFieldChange} />
                        </Grid>
                    </Grid>
                </Grid>
                <Grid item xs>
                    <Typography gutterBottom>
                        C
                    </Typography>
                    <Grid container>
                        <Grid item xs>
                            <Slider value={store.viewModel.frameTrace.circ} min={0} disabled onChange={handleCChange} />
                            <TextField value={store.viewModel.frameTrace.circ} disabled name={"circ"} onChange={handleFieldChange} />
                        </Grid>
                    </Grid>
                </Grid>
                <Grid item xs>
                    <Typography gutterBottom>
                        DBL
                    </Typography>
                    <Grid container>
                        <Grid item xs>
                            <Slider value={store.viewModel.frameTrace.dbl} min={0} onChange={handleDblChange} />
                            <TextField value={store.viewModel.frameTrace.dbl} name={"dbl"} onChange={handleFieldChange} />

                        </Grid>
                    </Grid>
                </Grid>
            </Grid>
        </>
    )
});