import * as React from 'react';
import {makeBooleanColumn, makeColumn, makeFormattedDateTimeColumn, ModelTable} from "@rx-suite/ui";
import {SyncStatusModel, SyncStatusStore} from "../../stores/SyncStatusStore";
import {currentUserCan} from "@rx-suite/stores";
import {t} from "@lingui/macro";
import {observer} from "mobx-react-lite";
import {SyncButton} from "../../components/sync_statuses/SyncButton";
import {SyncErrorButton} from "../../components/sync_statuses/SyncErrorButton";



export const SyncStatusTable = observer(() => {

    const columns = React.useMemo(() => {
        return [
            makeColumn("humanized_name", t`Name`, "string"),
            makeBooleanColumn("busy", t`In Progress`),
            makeBooleanColumn("failed", t`Failed`),
            makeFormattedDateTimeColumn("sync_started_at", t`Started At`, "dateTime"),
            makeFormattedDateTimeColumn("sync_ended_at", t`Ended At`, "dateTime")
        ]
    }, []);

    if(!currentUserCan("SyncStatus", "read")) {
        return null;
    }

    return(
        <ModelTable
            store={SyncStatusStore}
            columns={columns}
            customActions={(model) => (
                <>
                    <SyncButton model={model as SyncStatusModel} />
                    <SyncErrorButton model={model as SyncStatusModel} />
                </>

            )}
        />
    )
});