import {attribute, DataStore, model, Model} from "@rx-suite/data";
import {computed, reaction} from "mobx";
import {CurrentUserConfig} from "./ConfigStore";
import * as moment from "moment";

@model({
    transportLayer: "/api/customers",
    permissionTarget: "Customer",
})
export class CustomerModel extends Model {
    constructor() {
        super();
        super.setup(); // This registers the observable attributes and viewModel
    }

    @attribute()
    number = "";

    @attribute()
    name = "";

    @attribute()
    display_name = "";

    @attribute()
    ship_method_monday_id = "";

    @attribute()
    ship_method_tuesday_id = "";

    @attribute()
    ship_method_wednesday_id = "";

    @attribute()
    ship_method_thursday_id = "";

    @attribute()
    ship_method_friday_id = "";

    @attribute()
    ship_method_saturday_id = "";

    @attribute()
    ship_method_sunday_id = "";

    @computed
    get currentShippingMethodId() {
        const day = moment().format("dddd").toLowerCase();
        return this[`ship_method_${day}_id`];
    }
}


export const CustomerStore = CustomerModel.store as unknown as DataStore<CustomerModel>;

reaction(() => {
    return CurrentUserConfig.get().customer_ids;
},
() => {
    const config = CurrentUserConfig.get();
    CustomerStore.getByIds(config.customer_ids);
});