import {attribute, model, Model} from "@rx-suite/data";
import * as moment from 'moment';
import {computed} from "mobx";

@model({
    transportLayer: "/api/sync_statuses",
    permissionTarget: "SyncStatus",
    routingInfo: {
        list: '/maintenance',
    }
})
export class SyncStatusModel extends Model {
    static types = {
        sync_started_at: "date",
        sync_ended_at: "date",
        syncing: "boolean"
    }

    @attribute()
    sync_started_at = moment();

    @attribute()
    sync_ended_at = moment();

    @attribute()
    type = "";

    @attribute()
    failed = false;

    @attribute()
    error_message = "";

    @attribute()
    in_progress = false;

    @attribute()
    queued = false;

    @attribute()
    humanized_name = "";

    constructor() {
        super();
        super.setup(); // This registers the observable attributes and viewModel
    }

    @computed
    get busy() {
        return this.in_progress || this.queued;
    }

    async sync() {
        this.store.transportLayer.fetch(`/${this.getId()}/sync`, {
            method: "POST"
        })
    }
}

export const SyncStatusStore = SyncStatusModel.store;